/*******************************************************************************************
A step-by-step procedure to implement discrete choice experiments in Qualtrics 
Sylvain Weber, University of Neuchâtel, sylvain.weber@unine.ch
29 Mar 2019

Objective of this dofile: Import Qualtrics data and shape dataset
Input: FinalSurveyData.csv (dataset collected in FinalSurvey and exported using the legacy exporter)
Output: ShapedSurveyData.dta/csv (dataset shaped for future analyses)
*******************************************************************************************/

version 15
clear all
set more off


*** Import data collected in survey ***
#d ;
import delimited using FinalSurveyData.csv, clear 
	delim(",") varnames(1) case(lower) 
	stripquotes(yes) 
	encoding("utf-8")
;
#d cr


*** Data cleaning ***
drop in 1 // 1st observation contains the question text
ren v1 id // ResponseID from Qualtrics
keep id block ce_* // retain variables related to DCE only
ren ce_choicetask*_1 choice* // remove the initial "ce_", the final "_1", and rename all choicetask# variables to choice#
destring *, replace // all variables were stored as string; transformed to numeric


*** Reshape dataset from wide to long form ***
*Database has currently one observation per individual (wide form)
*Reshape database so that it has 8 observations per individual (long form)
reshape long choice, i(id) j(choicetask)

*Individuals assigned to block 1 received choice tasks 1-8
*Individuals assigned to block 2 received choice tasks 9-16
*-> The following observations are irrelevant (missing):
drop if (block==1 & !inrange(choicetask,1,8)) | (block==2 & !inrange(choicetask,9,16))


*** Reshape dataset to have 1 observation = 1 alternative ***
*Duplicate observations and create alt variable
expand 2, gen(alt)
replace alt = alt+1

*Create binary variable indicating which alternative was selected
gen selectedchoice = alt==choice


*** Include information from the experimental design ***
preserve
*Import experimental design from spreadsheet, alternative by alternative
forv i = 1/2 {
	#d ;
	import excel using ../StartFiles/ExpDesign.xlsx, clear 
		sheet(ExperimentalDesign) 
		cellrange(A1:L17)
		first
	;
	#d cr
	keep block choicetask A`i'_*
	gen alt = `i'
	ren A`i'_? att?
	tempfile alt`i'
	save `alt`i'', replace
}
restore

merge m:1 block choicetask alt using `alt1', nogen
merge m:1 block choicetask alt using `alt2', nogen update

*Rearrange variables and observations
order id block choicetask alt choice
sort id block choicetask alt


*** Label attributes and levels ***
*Attribute 1
la var att1 "Return airfare"
recode att1 (0=350) (1=450) (2=550) (3=650)
forv l = 350(100)650 {
	la def farelab `l' "$`l'", modify
}
la val att1 farelab

*Attribute 2
la var att2 "Total travel time, including stops"
replace att2 = att2+4
forv l = 4/7 {
	la def timelab `l' "`l'h", modify
}
la val att2 timelab

*Attribute 3
la var att3 "Food/beverage"
#d ;
la def foodlab 
	0 "none"
	1 "beverages only"
	2 "beverages + cold snack"
	3 "beverages + hot meal"
;
#d cr
la val att3 foodlab

*Attribute 4
la var att4 "Audio/Video entertainment"
#d ;
la def audiolab 
	0 "none"
	1 "audio only"
	2 "audio + short video clips"
	3 "audio + movie"
;
#d cr
la val att4 audiolab

*Attribute 5
la var att5 "Type of airplane"
#d ;
la def planelab 
	0 "Boeing 737"
	1 "Boeing 757"
	2 "Boeing 767"
	3 "Boeing 777"
;
#d cr
la val att5 planelab


*** Save final dataset ***
save ShapedSurveyData, replace // in Stata format
export delimited using ShapedSurveyData.csv, delim(",") replace // in csv format

exit
